/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.autotest;

import java.util.Vector;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.microedition.pim.PIM;
import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.PIMException;

import javax.microedition.pim.Contact;
import javax.microedition.pim.ContactList;

import com.funambol.blackberry.helper.VCardContactHelper;
import com.funambol.client.test.ContactsRobot;
import com.funambol.client.test.ContactsCommandRunner;
import com.funambol.client.test.CheckSyncClient;
import com.funambol.client.test.CheckSyncSource;

import com.funambol.syncml.spds.SyncItem;
import com.funambol.util.StringUtil;
import com.funambol.util.DateUtil;
import com.funambol.util.Log;

public class BlackBerryContactsRobot extends ContactsRobot {

    private static final String TAG_LOG = "BlackBerryContactsRobot";

    private Contact           currentContact = null;
    private ContactList       contactList    = null;
    private String            nameField[]    = null;
   
    public BlackBerryContactsRobot() {
    }

    public void createEmptyContact() throws Throwable {
        contactList = (ContactList) PIM.getInstance().openPIMList(PIM.CONTACT_LIST,
                                                                  PIM.READ_WRITE);
        currentContact = (Contact)contactList.createContact();
        nameField = null;
    }

    public void setContactField(String field, String value) throws Throwable {

        if (ContactsCommandRunner.CONTACT_FIELD_DISPLAY_NAME.equals(field)) {
            throw new ClientTestException("Display name not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_FIRST_NAME.equals(field)) {
            if (nameField == null) {
                nameField = new String[5];
            }
            nameField[Contact.NAME_GIVEN]  = value;
        } else if (ContactsCommandRunner.CONTACT_FIELD_LAST_NAME.equals(field)) {
            if (nameField == null) {
                nameField = new String[5];
            }
            nameField[Contact.NAME_FAMILY]  = value;
        } else if (ContactsCommandRunner.CONTACT_FIELD_MIDDLE_NAME.equals(field)) {
            throw new ClientTestException("Middle name not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_PREFIX_NAME.equals(field)) {
            if (nameField == null) {
                nameField = new String[5];
            }
            nameField[Contact.NAME_PREFIX] = value;
        } else if (ContactsCommandRunner.CONTACT_FIELD_SUFFIX_NAME.equals(field)) {
            if (nameField == null) {
                nameField = new String[5];
            }
            nameField[Contact.NAME_SUFFIX] = value;
        } else if (ContactsCommandRunner.CONTACT_FIELD_NICK_NAME.equals(field)) {
            throw new ClientTestException("Nick name not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_TEL_HOME.equals(field)) {
            currentContact.addString(Contact.TEL, Contact.ATTR_HOME, value);
        } else if (ContactsCommandRunner.CONTACT_FIELD_TEL_WORK.equals(field)) {
            currentContact.addString(Contact.TEL, Contact.ATTR_WORK, value);
        } else if (ContactsCommandRunner.CONTACT_FIELD_TEL_OTHER.equals(field)) {
            throw new ClientTestException("Phone other not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_TEL_OTHER2.equals(field)) {
            throw new ClientTestException("Phone other2 not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_TEL_CELL.equals(field)) {
            currentContact.addString(Contact.TEL, Contact.ATTR_MOBILE, value);
        } else if (ContactsCommandRunner.CONTACT_FIELD_TEL_PAGER.equals(field)) {
            currentContact.addString(Contact.TEL, Contact.ATTR_PAGER, value);
        } else if (ContactsCommandRunner.CONTACT_FIELD_TEL_FAX_HOME.equals(field)) {
            currentContact.addString(Contact.TEL, Contact.ATTR_FAX, value);
        } else if (ContactsCommandRunner.CONTACT_FIELD_TEL_FAX_WORK.equals(field)) {
            throw new ClientTestException("Fax work not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_TEL_COMPANY.equals(field)) {
            throw new ClientTestException("Phone company not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_TEL_OTHER_FAX.equals(field)) {
            throw new ClientTestException("Fax other not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_EMAIL_HOME.equals(field)) {
            currentContact.addString(Contact.EMAIL, Contact.ATTR_NONE, value);
        } else if (ContactsCommandRunner.CONTACT_FIELD_EMAIL_WORK.equals(field)) {
            throw new ClientTestException("Work email not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_EMAIL_OTHER.equals(field)) {
            throw new ClientTestException("Other email not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_EMAIL_IM.equals(field)) {
            throw new ClientTestException("IM not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_ADR_OTHER.equals(field)) {
            throw new ClientTestException("Other address not supported by BB");
        } else if (ContactsCommandRunner.CONTACT_FIELD_ADR_HOME.equals(field)) {
            String addr[] = createAddress(value);
            if (addr != null) {
                currentContact.addStringArray(Contact.ADDR, Contact.ATTR_HOME, addr);
            }
        } else if (ContactsCommandRunner.CONTACT_FIELD_ADR_WORK.equals(field)) {
            String addr[] = createAddress(value);
            if (addr != null) {
                currentContact.addStringArray(Contact.ADDR, Contact.ATTR_WORK, addr);
            }
        } else if (ContactsCommandRunner.CONTACT_FIELD_WEB.equals(field)) {
            currentContact.addString(Contact.URL, Contact.ATTR_NONE, value);
        } else if (ContactsCommandRunner.CONTACT_FIELD_WEB_HOME.equals(field)) {
            throw new ClientTestException("Unsupported field " + field);
        } else if (ContactsCommandRunner.CONTACT_FIELD_WEB_WORK.equals(field)) {
            throw new ClientTestException("Unsupported field " + field);
        } else if (ContactsCommandRunner.CONTACT_FIELD_BDAY.equals(field)) {
            Calendar date = DateUtil.parseDateTime(value);
            currentContact.addDate(Contact.BIRTHDAY, Contact.ATTR_NONE, date.getTime().getTime());
        } else if (ContactsCommandRunner.CONTACT_FIELD_ANNIVERSARY.equals(field)) {
            throw new ClientTestException("Unsupported field " + field);
        } else if (ContactsCommandRunner.CONTACT_FIELD_CHILDREN.equals(field)) {
            throw new ClientTestException("Unsupported field " + field);
        } else if (ContactsCommandRunner.CONTACT_FIELD_SPOUSE.equals(field)) {
            throw new ClientTestException("Unsupported field " + field);
        } else if (ContactsCommandRunner.CONTACT_FIELD_TITLE.equals(field)) {
            currentContact.addString(Contact.TITLE, Contact.ATTR_NONE, value);
        } else if (ContactsCommandRunner.CONTACT_FIELD_ORGANIZATION.equals(field)) {
            currentContact.addString(Contact.ORG, Contact.ATTR_NONE, value);
        } else if (ContactsCommandRunner.CONTACT_FIELD_NOTE.equals(field)) {
            currentContact.addString(Contact.NOTE, Contact.ATTR_NONE, value);
        } else if (ContactsCommandRunner.CONTACT_FIELD_PHOTO.equals(field)) {
            throw new ClientTestException("Unsupported field " + field);
        } else {
            throw new ClientTestException("Unsupported field " + field);
        }
    }

    public void loadContact(String firstName, String lastName) throws Throwable {
        currentContact = findContactByName(firstName, lastName);
        if (currentContact == null) {
            throw new ClientTestException("Cannot load contact: " + firstName + "," + lastName);
        }
    }

    public void saveContact() throws Throwable {
        if (nameField != null) {
            currentContact.addStringArray(Contact.NAME, PIMItem.ATTR_NONE, nameField);
        }
        currentContact.commit();
    }

    public void deleteContact(String firstName, String lastName) throws Throwable {
        Contact contact = findContactByName(firstName, lastName);
        contactList.removeContact(contact);    
    }

    public void deleteAllContacts() throws Throwable {
        Enumeration items = contactList.items();
        while(items.hasMoreElements()) {
            Contact contact = (Contact)items.nextElement();
            contactList.removeContact(contact);
        }
    }

    public void checkNewContact(String firstname, String lastname,
            CheckSyncClient client, boolean checkContent) throws Throwable
    {
        CheckSyncSource source = client.getSyncSource(
                                          CheckSyncClient.SOURCE_NAME_CONTACTS);

        String key = findContactKeyOnServer(firstname, lastname, client);

        Contact remoteContact = contactList.createContact();
        VCardContactHelper contactHelper = new VCardContactHelper(contactList);
        SyncItem item = new SyncItem(key);
        item = source.getItemContent(item);
        contactHelper.fillItem(remoteContact, new String(item.getContent()), null);
 

        Contact localContact = findContactByName(firstname, lastname);
        checkContact(localContact, remoteContact, checkContent);
    }

    public void checkUpdatedContact(String firstname, String lastname,
            CheckSyncClient client, boolean checkContent) throws Throwable
    {
    }

    public void checkDeletedContact(String firstname, String lastname, 
            CheckSyncClient client) throws Throwable
    {
    }

    public void checkNewContactOnServer(String firstname, String lastname,
            CheckSyncClient client, boolean checkContent) throws Throwable
    {
    }

    public void checkUpdatedContactOnServer(String firstname, String lastname,
            CheckSyncClient client, boolean checkContent) throws Throwable
    {
    }

    public void checkDeletedContactOnServer(String firstname, String lastname,
            CheckSyncClient client) throws Throwable
    {
    }

    public void loadContactOnServer(String firstName, String lastName,
            CheckSyncClient client) throws Throwable
    {
    }

    protected String getCurrentContactVCard() throws Throwable
    {
        VCardContactHelper contactHelper = new VCardContactHelper(contactList);
        return contactHelper.itemToString(currentContact, null);
    }

    private Contact findContactByName(String firstName, String lastName) throws Throwable {
        // Scan all contacts looking for the one that matched first and last
        // name
        Enumeration items = contactList.items();
        Contact c = null;
        while(items.hasMoreElements()) {
            Contact contact = (Contact)items.nextElement();
            int numValues = contact.countValues(Contact.NAME);
            if (numValues > 0) {
                StringBuffer name = new StringBuffer();

                String[] cName = contact.getStringArray(Contact.NAME, 0);
                String fName  = cName[Contact.NAME_GIVEN];
                String lName   = cName[Contact.NAME_FAMILY];
                if (firstName.equals(fName) && lastName.equals(lName)) {
                    c = contact;
                    break;
                }
            }
        }
        return c;
    }

    protected String findContactKeyOnServer(String firstName, String lastName,
            CheckSyncClient client) throws Throwable {

        Hashtable allItems = client.getSyncSource(CheckSyncClient.SOURCE_NAME_CONTACTS).getAllItems();

        Enumeration allElements = allItems.elements();
        VCardContactHelper contactHelper = new VCardContactHelper(contactList);
        
        while(allElements.hasMoreElements()) {
            SyncItem item = (SyncItem)allElements.nextElement();
            Contact contact = (Contact)contactList.createContact();
            contactHelper.fillItem(contact, new String(item.getContent()), null);
            String cName[] = contact.getStringArray(Contact.NAME, Contact.ATTR_NONE);
            if(firstName.equals(cName[Contact.NAME_GIVEN]) && lastName.equals(cName[Contact.NAME_FAMILY])) {
               return item.getKey();
            }
        }
        throw new ClientTestException("Can't find contact on server: " + firstName + " " + lastName);
    }

    private void checkContact(Contact localContact, Contact remoteContact,
            boolean checkContent) throws Throwable {
        if(checkContent) {
            assertEquals(localContact, remoteContact, "Contacts mismatch");
        }
    }


    private String[] createAddress(String value) {
        String arrayField[] = new String[7];

        // Break down the incoming value
        int idx = value.indexOf(';');
        int lastIdx = 0;
        int arrayIdx = 0;
        while(idx >=0) {
            int realIdx = -1;
            if (idx > 0 && value.charAt(idx - 1) != '\\') {
                realIdx = idx;
            } else if (idx == 0) {
                realIdx = idx;
            }

            if (realIdx >= 0) {
                String field = value.substring(lastIdx, realIdx);
                lastIdx = realIdx + 1;

                switch(arrayIdx) {
                    case 0:
                        arrayField[Contact.ADDR_POBOX] = field;
                        break;
                    case 1:
                        arrayField[Contact.ADDR_EXTRA] = field;
                        break;
                    case 2:
                        arrayField[Contact.ADDR_STREET] = field;
                        break;
                    case 3:
                        arrayField[Contact.ADDR_LOCALITY] = field;
                        break;
                    case 4:
                        arrayField[Contact.ADDR_REGION] = field;
                        break;
                    case 5:
                        arrayField[Contact.ADDR_POSTALCODE] = field;
                        break;
                    case 6:
                        arrayField[Contact.ADDR_COUNTRY] = field;
                        break;
                }
                arrayIdx++;
            }

            if (lastIdx < value.length()) {
                idx = value.indexOf(';', lastIdx);
            } else {
                idx = -1;
            }
        }
        if (arrayIdx > 0) {
            return arrayField;
        } else {
            return null;
        }
    }

    private void assertEquals(Contact c1, Contact c2, String msg) throws ClientTestException {

        String expectedStr = null;
        String resultStr = null;

        try {
            VCardContactHelper contactHelper = new VCardContactHelper(contactList);
            String c1VCard = contactHelper.itemToString(c1, null);
            String c2VCard = contactHelper.itemToString(c2, null);

            expectedStr = orderVCard(new String(c1VCard));
            resultStr   = orderVCard(new String(c2VCard));

            assertTrue(resultStr.equals(expectedStr), msg);
        } catch(Exception ex) {
            Log.error(TAG_LOG, "Expected: " + expectedStr + " -- Found: " + resultStr);
            throw new ClientTestException(msg);
        }
    }

}
