/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.util;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import net.rim.device.api.system.ApplicationDescriptor;
import net.rim.device.api.system.ApplicationManager;
import net.rim.device.api.ui.Font;
import net.rim.device.api.system.DeviceInfo;

import com.funambol.blackberry.BBVersion;
import com.funambol.blackberry.Constants;
import com.funambol.util.StringUtil;

import net.rim.device.api.i18n.Locale;
import java.util.Enumeration;

/**
 * Supports general using, language dictionary access, logging. Statics data and
 * static methods
 */
public class StaticDataHelper {

    public static final int     FONT_SIZE                      = 14;
    public static final int     FONT_STYLE                     = Font.PLAIN;
    
    public static final int     TITLE_FONT_SIZE                = 16;
    public static final int     TITLE_FONT_STYLE               = Font.BOLD;
    public static final int     TITLE_PADDING                  = 5;
    
    public static final int     SYNC_TYPE_TWO_WAY              = 0;
    public static final int     SYNC_TYPE_ONE_WAY_TO_SERVER    = 1;
    public static final int     SYNC_TYPE_ONE_WAY_TO_CLIENT    = 2;
    public static final int     SYNC_TYPE_REFRESH_TO_SERVER    = 3;
    public static final int     SYNC_TYPE_REFRESH_TO_CLIENT    = 4;

    private static final String LOG_TAG = "StaticDataHelper";
    
    private static final String XML_LANGUAGE_FILE           = "/xml/language";
    
    // Define the locale to use as default
    private static final String DEFAULT_LOCALE = BBVersion.DEFAULT_LOCALE;
    private static final String DEFAULT_LANGUAGE_FILE = XML_LANGUAGE_FILE +
            "_" + BBVersion.DEFAULT_LOCALE + ".xml";

    private static Hashtable languageHash = new Hashtable();

    private static String operatingSystem = null;

    // ----------------------------------------------------------------- Public
    // methods

    public static String getOS() {
        return operatingSystem;
    }

    public static String[] getOSVersion() {

        Log.trace(LOG_TAG, "getOSVersion");

        String os = getOS();
        Log.trace(LOG_TAG, "OS version string is: " + os);
        if (os == null) {
            return new String[0];
        }

        String[] tokens = StringUtil.split(os, ".");
        for(int i=0;i<tokens.length;++i) {
            Log.trace(LOG_TAG, "Ver["+i+"]=" + tokens[i]);
        }
        return tokens;
    }

    /**
     * Returns true iff the device has a dedicated key to open the menu. The
     * implementation is based on the device model and it works for the models
     * currently supported by our APIs (>= 4.2.1)
     */
    public static boolean hasDedicatedMenuKey() {
        String deviceName = DeviceInfo.getDeviceName();
        if (deviceName.startsWith("7130")  ||
            deviceName.startsWith("8700")  ||
            deviceName.startsWith("8703")  ||
            deviceName.startsWith("8705")  ||
            deviceName.startsWith("8707")) {

            return false;
        } else {
            return true;
        }
    }
    
    /**
     * Make a String by value of <i>tag</i>.
     * 
     * @param key
     *            key to find
     * @return key value
     */
    public static String getLanguage(String key) {
        Object valueFromHash = StaticDataHelper.languageHash.get(key);
        if (valueFromHash != null) {
            return (String) valueFromHash;
        }
        // This should never happen
        System.out.println("Language string missing for key: " + key);
        return "";
    }

    public static void init(Class c) throws Exception {
        loadVersion();
        loadLanguage(c);
        setOS();
    }

    /**
     * Must be called on initialization
     */
    public static void loadVersion() {
        String version = BBVersion.VERSION;
        Constants.VER = version;
        Log.info(LOG_TAG, "Version: " + version);
    }

    /**
     * Load the application language
     */
    private static void loadLanguage(Class c) throws Exception {

        Log.info(LOG_TAG, "Loading default language: " + DEFAULT_LOCALE);

        //
        // Initialize the default language
        //
        if(!includeLanguageFile(DEFAULT_LANGUAGE_FILE, DEFAULT_LOCALE, c)) {
            Log.error(LOG_TAG, "Can't load default language file: " + DEFAULT_LANGUAGE_FILE);
            throw new Exception("Can't load default language: " + DEFAULT_LANGUAGE_FILE);
        }

        //
        // Search for customizations of the default language
        //
        // Example: "/xml/language_en_US_custom.xml"
        //
        String customDefaultLanguageFile = XML_LANGUAGE_FILE + "_" + DEFAULT_LOCALE +
                "_custom.xml";
        includeLanguageFile(customDefaultLanguageFile, DEFAULT_LOCALE, c);
        
        //
        // Read the current Locale information: 
        //  - l_language (e.g. "en")
        //  - l_country  (e.g. "US")
        //
        Locale locale = Locale.getDefault();
        String l_language = locale.getLanguage();
        String l_country  = locale.getCountry();

        Log.info(LOG_TAG, "Device locale language: " + l_language);
        Log.info(LOG_TAG, "Device locale country: "  + l_country);

        //
        // Search for a language resource related to current Locale, giving the
        // following priorities:
        //
        // 1) "/xml/language_" + language + "_" + country + ".xml"
        // 2) "/xml/language_" + language + ".xml"
        //
        String languageFile = null;
        String localeInfo   = null;

        boolean localeFound = false;
        String  localeFoundString = null;

        //
        // Search for "/xml/language_" + language + "_" + country + ".xml"
        //
        if(!StringUtil.isNullOrEmpty(l_language) && !StringUtil.isNullOrEmpty(l_country)) {
            // Search for "/xml/language_" + l_language + "_" + l_country + ".xml"
            localeInfo = l_language + "_" + l_country;
            languageFile = XML_LANGUAGE_FILE + "_" + localeInfo + ".xml";
            if(includeLanguageFile(languageFile, localeInfo, c)) {
                localeFound = true;
                localeFoundString = localeInfo;
            }
        }

        //
        // Search for "/xml/language_" + language + ".xml"
        //
        if(!localeFound && !StringUtil.isNullOrEmpty(l_language)) {
            // Search for "/xml/language_" + l_language + ".xml"
            localeInfo = l_language;
            languageFile = XML_LANGUAGE_FILE + "_" + localeInfo + ".xml";
            if(includeLanguageFile(languageFile, localeInfo, c)) {
                localeFound = true;
                localeFoundString = localeInfo;
            }
        }

        //
        // Search for customizations of the locale language
        //
        // Example: "/xml/language_de_custom.xml"
        //
        if(localeFound && localeFoundString != null) {
            // Search for customizations of the locale language
            String customLocaleLanguageFile = XML_LANGUAGE_FILE + "_" + localeFoundString +
                    "_custom.xml";
            includeLanguageFile(customLocaleLanguageFile, localeFoundString, c);
        }
    }

    /**
     * Include the given language file, if available, into the languageHash.
     *
     * @param languageFile The language file name
     * @param locale the language locale (e.g. en_US, de)
     *
     * @param c the Class object used to retrieve resource files.
     * 
     * @return True if the given language file exists and has been filled into
     * the language Hashtable (languageHash)
     */
    private static boolean includeLanguageFile(String languageFile,
            String locale, Class c) throws Exception {

        InputStream languageStream = c.getResourceAsStream(languageFile);
        if(languageStream != null) {
            Log.info(LOG_TAG, "Found language file: " + languageFile);
            try {
                if(DEFAULT_LANGUAGE_FILE.equals(languageFile)) {
                    if(languageHash.isEmpty()) {
                        // Default language must be initialized
                        languageHash = readLanguageStream(languageStream, locale);
                        Log.info(LOG_TAG, "Reading default language file: " +
                                languageFile + " successfully");
                        return true;
                    } else {
                        // The default language has been already loaded
                        Log.info("Default language already read.");
                        return false;
                    }
                }
                Hashtable language_ht = readLanguageStream(languageStream, locale);
                Log.info(LOG_TAG, "Reading language file: " + languageFile +
                        " successfully");

                // Merge with default language
                Enumeration keys = language_ht.keys();
                while(keys.hasMoreElements()) {
                    String key = (String)keys.nextElement();
                    languageHash.put(key, (String)language_ht.get(key));
                }
                return true;
            } catch (final Throwable e) {
                e.printStackTrace();
                Log.error(LOG_TAG, "Error reading language file: " + languageFile +
                        " Exception: " + e.toString());
                throw new Exception("Error reading language file: " + languageFile);
            }
        } else {
            Log.debug(LOG_TAG, "Language file not found: " + languageFile);
            return false;
        }
    }

    /**
     * Read the given InputStream and return the corresponding language
     * Hashtable.
     * 
     * @param stream
     * @param locale
     * @return
     * @throws Throwable
     */
    private static Hashtable readLanguageStream(InputStream stream, String locale)
            throws Throwable {
        String language = read(stream);
        return new SimpleXMLParser().parseData(locale, language);
    }

    private static void setOS() {
        final ApplicationDescriptor[] appDes = ApplicationManager.getApplicationManager()
                .getVisibleApplications();
        String version = "";
        for (int i = appDes.length - 1; i >= 0; --i) {
            if (appDes[i].getModuleName().equals("net_rim_bb_ribbon_app")) {
                version = appDes[i].getVersion();
                break;
            }
        }
        operatingSystem = version;
    }

    /**
     * Reads the content of the given input stream.
     * 
     * @param is the input stream
     */
    public static String read(InputStream is) throws IOException {
        final StringBuffer sb = new StringBuffer();
        try {
            final byte[] buf = new byte[1024];
            int nbyte = -1;
            while ((nbyte = is.read(buf)) >= 0) {
                sb.append(new String(buf, 0, nbyte));
            }
        } finally {
            is.close();
        }
        return sb.toString();
    }
}
